// colorchange.cpp
// by Glenn G. Chappell
// October 2003
//
// For CS 381
// Example program for Button class (button.h/button.cpp)

#include <iostream>
using std::cerr;
using std::endl;
#include <stdlib.h>  // Some versions of MS-Vis C++ have broken <cstdlib>
//using std::exit;
#include <GL/glut.h> // GLUT stuff - includes OpenGL headers as well

#include "button.h"  // for class Button


// Global variables
// Window/viewport
const int startwinsize = 400; // Starting window width & height, in pixels

// Keyboard
const int ESCKEY = 27;        // ASCII value of escape character

// For color-changing rectangle
GLdouble color_r = 0.5;  // r, g, b components of rectangle color.
GLdouble color_g = 0.5;  //  Initial color is gray.
GLdouble color_b = 0.5;

// Buttons
Button redbutton;     // Sets color to red
Button yellowbutton;  // Sets color to yellow
Button bluebutton;    // Sets color to blue


// redbuttonaction
// Does action for red button: set color to red
void redbuttonaction()
{
   color_r = 0.9;
   color_g = 0.1;
   color_b = 0.1;
   glutPostRedisplay();
}


// yellowbuttonaction
// Does action for yellow button: set color to yellow
void yellowbuttonaction()
{
   color_r = 0.9;
   color_g = 0.9;
   color_b = 0.1;
   glutPostRedisplay();
}


// bluebuttonaction
// Does action for blue button: set color to blue
void bluebuttonaction()
{
   color_r = 0.1;
   color_g = 0.1;
   color_b = 0.9;
   glutPostRedisplay();
}


// display
// The GLUT display function
void display()
{
   glClear(GL_COLOR_BUFFER_BIT);

   // Draw the colored rectangle
   glColor3d(color_r, color_g, color_b);
   glRectd(-0.7, 0.0, 0.7, 0.5);

   // Draw the buttons
   redbutton.display();
   yellowbutton.display();
   bluebutton.display();

   glutSwapBuffers();
}


// mouse
// The GLUT mouse function
void mouse(int button, int state, int x, int y)
{
   // Call buttons' mouse handlers
   bool handled = false;  // True if this event has been handled by a button
   if (!handled) handled = redbutton.handlemouse(button, state, x, y);
   if (!handled) handled = yellowbutton.handlemouse(button, state, x, y);
   if (!handled) handled = bluebutton.handlemouse(button, state, x, y);
}


// motion
// The GLUT motion function
void motion(int x, int y)
{
   // Call buttons' mouse-motion handlers
   bool handled = false;  // True if this event has been handled by a button
   if (!handled) handled = redbutton.handlemotion(x, y);
   if (!handled) handled = yellowbutton.handlemotion(x, y);
   if (!handled) handled = bluebutton.handlemotion(x, y);
}


// reshape
// The GLUT reshape function
void reshape(int w, int h)
{
    glViewport(0, 0, w, h);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    // We set up coordinate system so that aspect ratios are always correct,
    //  and the region from -1..1 in x & y always just fits in the viewport.
    if (w > h)
    {
       gluOrtho2D(-double(w)/h, double(w)/h, -1., 1.);
    }
    else
    {
       gluOrtho2D( -1., 1., -double(h)/w, double(h)/w);
    }

    glMatrixMode(GL_MODELVIEW);  // Always go back to modelview mode
}


// keyboard
// The GLUT keyboard function
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
   case ESCKEY:  // ESC: Quit
      exit(0);
      break;
   }
}


// idle
// The GLUT idle function
void idle()
{
   // Print OpenGL errors, if there are any (for debugging)
   if (GLenum err = glGetError())
   {
      cerr << "OpenGL ERROR: " << gluErrorString(err) << endl;
   }
}


// init
// Initialization
// Called by main
void init()
{
   glClearColor(1.0, 1.0, 1.0, 0.0);

   redbutton.setpos(-0.8, -0.5);
   redbutton.setsize(0.4, 0.2);
   redbutton.setlabel("Red");
   redbutton.setaction(redbuttonaction);
   redbutton.setactive(true);

   yellowbutton.setpos(-0.2, -0.5);
   yellowbutton.setsize(0.4, 0.2);
   yellowbutton.setlabel("Yellow");
   yellowbutton.setaction(yellowbuttonaction);
   yellowbutton.setactive(true);

   bluebutton.setpos(0.4, -0.5);
   bluebutton.setsize(0.4, 0.2);
   bluebutton.setlabel("Blue");
   bluebutton.setaction(bluebuttonaction);
   bluebutton.setactive(true);
}


int main(int argc, char ** argv)
{
   // Initialize OpenGL/GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);

   // Make a window
   glutInitWindowSize(startwinsize, startwinsize);
   glutInitWindowPosition(50, 50);
   glutCreateWindow("CS 381 - Button Example Program");

   // Initialize GL states & register callbacks
   init();
   glutDisplayFunc(display);
   glutMouseFunc(mouse);
   glutMotionFunc(motion);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   glutIdleFunc(idle);

   // Do something
   glutMainLoop();

   return 0;
}
